-- LayoutOptions.lua : Options config
if select(6, GetAddOnInfo("PitBull4_" .. (debugstack():match("[o%.][d%.][u%.]les\\(.-)\\") or ""))) ~= "MISSING" then return end

local _G = getfenv(0)
local PitBull4 = _G.PitBull4
local PitBull4_VisualHelpers= PitBull4:GetModule("VisualHelpers")
local L = PitBull4.L
local GetLayoutDB = PitBull4.Options.GetLayoutDB

function PitBull4_VisualHelpers:CreateVisualHelperIndicator(name)

	local full_name = "VisualHelpers: ".. name
	local VisualHelperIndicator = PitBull4:NewModule(full_name)
	VisualHelperIndicator.full_name = full_name

	local border_path = "Interface\\AddOns\\PitBull4_VisualHelpers\\border"
	
	VisualHelperIndicator:SetModuleType("indicator")
	VisualHelperIndicator:SetName(L[full_name])
	VisualHelperIndicator:SetDescription(L["Visual Helper for '%s'."]:format(name))
	VisualHelperIndicator:SetDefaults({
		attach_to = "HealthBar",
		location = "center",
		position = 1,
		size = 1.6,
		solid = false,
		usetexture = false,
		texture = nil,
		color = { 1, 1, 1, 1 },
		border = true,
		bordercolor = {0, 0, 0, 1},
		cooldown = true,
		count = true,
		only_self = false,
	}, nil)

	-- :::::::::::: LOCALS ::::::::::::: -------------------------------------------------------------------------

	local LibSharedMedia = LibStub("LibSharedMedia-3.0", true)
	LoadAddOn("AceGUI-3.0-SharedMediaWidgets")
	local AceGUI = LibStub("AceGUI-3.0")
	
	-- :::::::::::: VisualHelperIndicator ::::::::::::: -------------------------------------------------------------------------

	local db = GetLayoutDB(VisualHelperIndicator)

	function VisualHelperIndicator:ClearFrame(frame)
		if not frame[full_name] then
			return false
		end

		local f = frame[full_name]
		f.texture = f.texture:Delete()
		f.border = f.border:Delete()
		f.cooldown = f.cooldown:Delete()
		f.overlay = f.overlay:Delete()
		f.count = f.count:Delete()
		f.unit = nil
		frame[full_name] = f:Delete()

		--print(full_name .. ' --> cleared')
		return true
	end

	function VisualHelperIndicator:UpdateFrame(frame)
		--print(full_name .. ' --> try update --> '.. frame.unit .. " = " .. UnitGUID(frame.unit))
		if not frame[full_name..'_show'] then
			--print(full_name .. ' --> clear')
			return self:ClearFrame(frame)
		end

		local f = frame[full_name]

		--if f then
			--print(full_name .. ' --> update')
		--	return false
		--else
			
			local aura = frame[full_name..'_aura']
			local guid = frame[full_name..'_guid']
			if guid~=UnitGUID(frame.unit) then
				-- This unit needs an update!
				return false
			end

			if not f then f = PitBull4.Controls.MakeFrame(frame) end
			frame[full_name] = f
			f:SetFrameLevel(frame:GetFrameLevel() + 20)

			f:SetHeight(15)
			f:SetWidth(15)
			
			local texture 
			if not f.texture then
				texture = PitBull4.Controls.MakeTexture(f, "BACKGROUND")
				texture:SetWidth(15)
				texture:SetHeight(15)
				texture:SetPoint("CENTER", f, "CENTER", 0, 0)
				f.texture = texture
			else
				texture = f.texture
			end
					
			if db.solid then
				texture:SetTexture([[Interface\Buttons\WHITE8X8]])
				texture:SetVertexColor(unpack(db.color))
			else
				if db.usetexture then
					if LibSharedMedia then
						texture:SetTexture(LibSharedMedia:Fetch("statusbar", db.texture or "Blizzard"))
					else
						texture:SetTexture([[Interface\TargetingFrame\UI-StatusBar]])
					end
					texture:SetVertexColor(unpack(db.color))
				else
					texture:SetTexture(aura.icon)
					texture:SetVertexColor(1, 1, 1, 1)
				end
			end
			
			local border
			if not f.border then
				border = PitBull4.Controls.MakeTexture(f, "BORDER")
				border:SetTexture(border_path)
				border:SetAllPoints(f)
				f.border = border
			else
				border = f.border
			end
			
			if db.border then
				border:Show()
				border:SetVertexColor(unpack(db.bordercolor))
			else
				border:Hide()
				texture:SetAllPoints(f)
			end
							
			local cooldown
			if not f.cooldown then
				cooldown = PitBull4.Controls.MakeCooldown(f)
				f.cooldown = cooldown
				cooldown:SetReverse(true)
				cooldown:SetFrameLevel(f:GetFrameLevel() + 1)
				cooldown:SetAllPoints(f)
			else
				cooldown = f.cooldown
			end
			
			local overlay
			if not f.overlay then
				overlay = PitBull4.Controls.MakeFrame(f, "OVERLAY")
				f.overlay = overlay
				overlay:SetAllPoints(control)
			else
				overlay = f.overlay
			end
			
			local count
			if not f.count then
				count = PitBull4.Controls.MakeFontString(overlay, "OVERLAY")
				f.count = count
				count:SetFont(STANDARD_TEXT_FONT, 8, "OUTLINE")
				count:SetShadowColor(0, 0, 0, 1)
				count:SetShadowOffset(0.8, -0.8)
				count:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 1, 1)
			else
				count = f.count
			end
			
			if db.cooldown and (aura.duration and aura.duration > 0) then
				cooldown:Show()
				cooldown:SetCooldown(aura.expirationTime - aura.duration, aura.duration)
			else
				cooldown:Hide()
			end
			
			count:SetText((aura.count > 1 and db.count) and aura.count or "")
			
			f:Show()
			--print(full_name .. ' --> create')
		
			return true
		--end
	end

	VisualHelperIndicator:SetLayoutOptionsFunction(function(self)

		local function get_new_visual_helper_options()
			return {
				solid = false,
				solidcolor = { 1, 1, 1, 1 },
				border = true,
				bordercolor = {0, 0, 0, 1},
				cooldown = true,
			}
		end
		local function ensure_layout_db()
			if not db then
				db = get_new_visual_helper_options()
			end
		end
		local function get_layout(info)
			local id = info[#info]
			return db[id]
		end
		local function set_layout(info, value)
			local id = info[#info]
			db[id] = value
			for frame in PitBull4:IterateFrames() do
				self:Clear(frame)
			end
			PitBull4_VisualHelpers:UpdateIndicators()
		end                              

		ensure_layout_db()

		return
		'h1', {
			type = 'header',
			name = '',
			order = 9,
		},
		
		'solid', {
			type = 'toggle',
			name = L['Solid Color'],
			desc = L['Visual helper will be rendered as solid color.'],
			get = get_layout,
			set = set_layout,
			disabled = function(info)
				return self.enabled
			end,
			order = 10,
		},
		
		'usetexture', {
			type = 'toggle',
			name = L['Use texture'],
			desc = L['Use a texture for this helper'],
			get = get_layout,
			set = set_layout,
			hidden = function(info)
				return db.solid
			end,
			disabled = function(info)
				return self.enabled
			end,
			order = 11,
		},
		'texture', {
			type = 'select',
			name = L["Texture"],
			desc = L["What texture to use for this visual helper."] .. "\n" .. L["If you want more textures, you should install the addon 'SharedMedia'."],
			get = get_layout,
			set = set_layout,
			values = function(info)
				return LibSharedMedia:HashTable("statusbar")
			end,
			hidden = function(info)
				return not LibSharedMedia or db.solid or not db.usetexture
			end,
			dialogControl = AceGUI.WidgetRegistry["LSM30_Statusbar"] and "LSM30_Statusbar" or nil,
			disabled = function(info)
				return self.enabled
			end,
			order = 12,
		},   	
		
		'color', {
			type = 'color',
			name = L['Color'],
			desc = L["Color for this visual helper."],
			get = function(info)
				return unpack(get_layout(info))
			end,
			set = function(info, r, g, b, a)
				set_layout(info, {r, g, b, a})
			end,
			hidden = function(info)
				return (not db.solid) and (not db.usetexture)
			end,
			disabled = function(info)
				return self.enabled
			end,
			order = 13,
		},
		
		'border', {
			type = 'toggle',
			name = L['Border'],
			desc = L['Visual helper will have a border.'],
			get = get_layout,
			set = set_layout,
			disabled = function(info)
				return self.enabled
			end,
			order = 14,
		},
		'bordercolor', {
			type = 'color',
			name = L['Border Color'],
			desc = L["Color for this visual helper's border."],
			get = function(info)
				return unpack(get_layout(info))
			end,
			set = function(info, r, g, b, a)
				set_layout(info, {r, g, b, a})
			end,
			hidden = function(info)
				return not db.border
			end,
			disabled = function(info)
				return self.enabled
			end,
			order = 15,
		},
		
		'cooldown', {
			type = 'toggle',
			name = L['Cooldown Sweep'],
			desc = L['Visual helper will have a cooldown sweep overlay.'],
			get = get_layout,
			set = set_layout,
			disabled = function(info)
				return self.enabled
			end,
			order = 16,
		},
		'count', {
			type = 'toggle',
			name = L['Stack Count'],
			desc = L['Visual helper will have a stack count visible.'],
			get = get_layout,
			set = set_layout,
			disabled = function(info)
				return self.enabled
			end,
			order = 17,
		}
		
	end)

	return VisualHelperIndicator

end